REM
REM ----------------------------------------
REM  This file is part of the zBoy project.
REM  Copyright (C) Mateusz Viste 2010
REM ----------------------------------------
REM
REM Support for battery-saved RAM. This file
REM provides two routines:
REM
REM SaveBattRam()  - Saves the battery-saved RAM
REM LoadBattRAM()  - Loads the battery-saved RAM
REM
REM The battery-saved RAM is stored in the
REM MemoryBankedRAM(0 TO 2097152) table. Battery
REM support can be checked in RomInfos.Battery
REM (0=No battery ; 1=Battery), and RAM size can
REM be checked via RomInfos.RamSize (in bytes).
REM
REM zBoy is saving the content of battery-saved
REM RAM in a specific (platform-dependant) location:
REM Linux -> $home/.zboy/[crc32].ram
REM Win32 -> $USERPROFILE + "\zboy_sav\[crc32].ram"
REM  Note: if $USERPROFILE is not defined, fallback to DOS method.
REM DOS   -> EXEPATH() + "\battmem\[crc32].ram"
REM

#DEFINE SaveBattFile() (HEX(RomInfos.CRCsum, 8) & ".ram")
#DEFINE SaveStateFile() (HEX(RomInfos.CRCsum, 8) & ".sav")

SUB LoadBattRAM()
  DIM AS UINTEGER x = 0, BattFileHandler
  GetSaveDir()
  IF FileExists(SaveDir & SaveBattFile()) <> 0 THEN
    BattFileHandler = FREEFILE()
    OPEN SaveDir & SaveBattFile() FOR BINARY AS BattFileHandler
    IF (LOF(BattFileHandler) = RomInfos.RamSize) AND (RomInfos.RamSize > 0) THEN
      FOR x = 1 TO RomInfos.RamSize
        GET #BattFileHandler, x, MemoryBankedRAM(&hA000 + x - 1)
      NEXT
    END IF
    CLOSE BattFileHandler
  END IF
END SUB


SUB SaveBattRAM()
  DIM AS UINTEGER x = 0, BattFileHandler
  GetSaveDir()
  IF IsDirectory(SaveDir) = 0 THEN
    MKDIR SaveDir
  END IF
  BattFileHandler = FREEFILE()
  OPEN SaveDir & SaveBattFile() FOR OUTPUT AS BattFileHandler
  FOR x = 1 TO RomInfos.RamSize
    PUT #BattFileHandler, x, MemoryBankedRAM(&hA000 + x - 1)
  NEXT x
  CLOSE BattFileHandler
END SUB
