REM
REM  Set of microinstructions to be used by the Z80 emulation core
REM  This file is part of the zBoy project
REM  Copyright (C) Mateusz Viste 2010
REM


'#MACRO WriteRegAF(x)
'  Register.F = (x AND &b11111111)
'  Register.A = ((x SHR 8) AND &b11111111)
'#ENDMACRO
#DEFINE ReadRegAF() (DwordVal(Register.F, Register.A))

#MACRO WriteRegBC(x, y)
  Register.B = x
  Register.C = y
#ENDMACRO
#DEFINE ReadRegBC() (DwordVal(Register.C, Register.B))

#MACRO WriteRegDE(x, y)
  Register.D = x
  Register.E = y
#ENDMACRO
#DEFINE ReadRegDE() (DwordVal(Register.E, Register.D))

#MACRO WriteRegHL(x, y)
  Register.H = x
  Register.L = y
#ENDMACRO
#DEFINE ReadRegHL() (DwordVal(Register.L, Register.H))


#MACRO SetFlagZ() ' F struct: "ZNHC0000"
  Register.F = (Register.F OR &b10000000)
#ENDMACRO
#MACRO ResetFlagZ()
  Register.F = (Register.F AND &b01111111)
#ENDMACRO
#DEFINE GetFlagZ() ((Register.F AND &b10000000) SHR 7)

#MACRO SetFlagN() ' F struct: "ZNHC0000"
  Register.F = (Register.F OR &b01000000)
#ENDMACRO
#MACRO ResetFlagN()
  Register.F = (Register.F AND &b10111111)
#ENDMACRO
#DEFINE GetFlagN() ((Register.F AND &b01000000) SHR 6)

#MACRO SetFlagH() ' F struct: "ZNHC0000"
  Register.F = (Register.F OR &b00100000)
#ENDMACRO
#MACRO ResetFlagH()
  Register.F = (Register.F AND &b11011111)
#ENDMACRO
#DEFINE GetFlagH() ((Register.F AND &b00100000) SHR 5)

#MACRO SetFlagC() ' F struct: "ZNHC0000"
  Register.F = (Register.F OR &b00010000)
#ENDMACRO
#MACRO ResetFlagC()
  Register.F = (Register.F AND &b11101111)
#ENDMACRO
#DEFINE GetFlagC() ((Register.F AND &b00010000) SHR 4)

#MACRO PushToStack(TmpRegister1, TmpRegister2)    ' Push a register pair to the stack
  Register.SP -= 1  ' decrement SP to update the Stack Point address
  MemoryWrite(Register.SP, TmpRegister1)        ' Write the byte
  Register.SP -= 1  ' decrement SP to update the Stack Point address
  MemoryWrite(Register.SP, TmpRegister2)        ' Write the byte
#ENDMACRO

#MACRO PopFromStack(popreg1, popreg2)    ' Pop a register pair from the stack
  IF (Register.SP < &hFFFE) THEN  ' If stack pointer not over available memory...
      MemoryRead(Register.SP, popreg2)
      Register.SP += 1 ' increments SP to update the Stack Point address
      MemoryRead(Register.SP, popreg1)
      Register.SP += 1 ' increments SP to update the Stack Point address
    ELSE
      #IFDEF DEBUGMODE
        DebugOnScreen()
        DebugShowCpuRegisters()
      #ENDIF
      PrintMsg("FATAL ERROR: STACK OVERFLOW!")
      PressAnyKey()
      END
  END IF
#ENDMACRO

#MACRO IncReg8(RegToInc)
  ResetFlagN()
  IF (RegToInc = 255) THEN
      RegToInc = 0
      SetFlagZ()
    ELSE
      RegToInc += 1
      ResetFlagZ()
  END IF
  IF ((RegToInc AND &b1111) = 0) THEN
      SetFlagH()
    ELSE
      ResetFlagH()
  END IF
#ENDMACRO


SUB DecReg8(BYREF RegToInc AS UBYTE) ' Decrements by 1 an 8bit register
  STATIC OldRegToInc AS UBYTE
  OldRegToInc = RegToInc
  SetFlagN()
  IF RegToInc = 0 THEN
      RegToInc = 255
    ELSE
      RegToInc -= 1
  END IF
  IF (OldRegToInc AND &b1111) < (RegToInc AND &b1111) THEN
      SetFlagH()
    ELSE
      ResetFlagH()
  END IF
  IF RegToInc = 0 THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
END SUB


#MACRO IncReg16(x, y) ' Increment a 16 bit register (pair of 8bit registers)
  IF ((x = 255) AND (y = 255)) THEN  ' IF Reg = 65535 then Reg = 0
      x = 0
      y = 0
    ELSE
      IF (y = 255) THEN
          y = 0
          x += 1
        ELSE
          y += 1
      END IF
  END IF
#ENDMACRO


#MACRO DecReg16(x, y) ' Decrement a 16 bit register (pair of 8bit registers)
  IF ((x = 0) AND (y = 0)) THEN  ' IF Reg = 0 then Reg = 65535
      x = 255
      y = 255
    ELSE
      IF (y = 0) THEN
          y = 255
          x -= 1
        ELSE
          y -= 1
      END IF
  END IF
#ENDMACRO


'SUB SubValFromReg8(BYREF RegToSub AS UBYTE, BYREF SubValue AS UBYTE)  ' Subtract a n ubyte value from a 8bit register
#MACRO SubValFromReg8(RegToSub, SubValue)
  SetFlagN()
  IF (RegToSub = SubValue) THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  IF ((RegToSub AND &b1111) < (SubValue AND &b1111)) THEN
      SetFlagH()
    ELSE
      ResetFlagH()
  END IF
  IF (RegToSub < SubValue) THEN
      RegToSub = (256 - (SubValue - RegToSub))
      SetFlagC()
    ELSE
      RegToSub -= SubValue
      ResetFlagC()
  END IF
'END SUB
#ENDMACRO


#MACRO AddToA(ValToAdd)  ' Add ValToAdd to register A and sets some flags (8bit arithmetic)
    IF ((Register.A AND &b1111) + (ValToAdd AND &b1111)) > 15 THEN
        SetFlagH()
      ELSE
        ResetFlagH()
    END IF
    IF (Register.A + ValToADD) > 255 THEN
        SetFlagC()
      ELSE
        ResetFlagC()
    END IF
    Register.A = ((Register.A + ValToAdd) AND &b11111111)
    IF (Register.A = 0) THEN
        SetFlagZ()
      ELSE
        ResetFlagZ()
    END IF
    ResetFlagN()
#ENDMACRO


SUB AddToHL(BYREF ValToAdd AS INTEGER)  ' Add ValToAdd to register HL and sets some flags (16bit Z80 arithmetic)
    STATIC AS UINTEGER TempHLvalue
    TempHLvalue = ReadRegHL()
    REM I'm not touching H flag, but really not sure what should I do...
    REM The Z80 manual says "Set if carry from bit 11", but NO$GMB doesn't set it :-/
    'IF (TempHLvalue AND &b111111111111) + (ValToAdd AND &b111111111111) > 4095 THEN SetFlagH() ELSE ResetFlagH()
    IF (TempHLvalue + ValToADD) > 65535 THEN
        SetFlagC()
      ELSE
        ResetFlagC()
    END IF
    TempHLvalue = (TempHLvalue + ValToAdd) AND &b1111111111111111
    WriteRegHL((TempHLvalue SHR 8), (TempHLvalue AND &b11111111))   ' Write 16bit result into HL
    ResetFlagN()
END SUB


SUB AddToSP(BYREF ByteToAdd AS BYTE)  ' Add ByteToAdd signed byte to register SP and set some Z80 flags
  STATIC AS INTEGER ResultAddToSP
  ResultAddToSP = ((Register.SP + ByteToAdd) AND &hFFFF)
  IF (ByteToAdd > 0) THEN
      IF (Register.SP > ResultAddToSP) THEN
          SetFlagC()
        ELSE
          ResetFlagC()
      END IF
    ELSE
      IF (Register.SP < ResultAddToSP) THEN
          SetFlagC()
        ELSE
          ResetFlagC()
      END IF
  END IF
  IF (((Register.SP XOR ByteToAdd XOR ResultAddToSP) AND 4096) > 0) THEN
      SetFlagH()
    ELSE
      ResetFlagH()
  END IF
  Register.SP = ResultAddToSP
  ResetFlagZ()
  ResetFlagN()
END SUB


#MACRO CmpA(n)
  IF (Register.A AND 15) < (n AND 15) THEN
      SetFlagH()
    ELSE
      ResetFlagH()
  END IF
  IF (Register.A < n) THEN
      SetFlagC()
    ELSE
      ResetFlagC()
  END IF
  IF (Register.A = n) THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  SetFlagN()
#ENDMACRO


#MACRO XorA(xorvalz)
  REM  Xor A with xorvalz, saves results into A, and sets some Z80 flags.
  ResetFlagN()
  ResetFlagH()
  ResetFlagC()
  Register.A = (Register.A XOR xorvalz)
  IF (Register.A = 0) THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
#ENDMACRO


#MACRO AndA(ValToAndWith)
  Register.A = (Register.A AND ValToAndWith)
  IF (Register.A = 0) THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagN()
  SetFlagH()
  ResetFlagC()
#ENDMACRO


#MACRO OrA(ValToOrWith) ' A = A OR ValToOrWith (and sets some Z80 flags)
  Register.A = (Register.A OR ValToOrWith)
  IF (Register.A = 0) THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagN()
  ResetFlagH()
  ResetFlagC()
#ENDMACRO


#MACRO SwapUbyte(ubytevalue)
  REM  Swaps upper & lower nibbles of ubytevalue and sets some Z80 flags
  ubytevalue = ((ubytevalue AND &b11110000) SHR 4) OR ((ubytevalue AND &b00001111) SHL 4)
  IF (ubytevalue = 0) THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagN()
  ResetFlagH()
  ResetFlagC()
#ENDMACRO


#MACRO TestBit0(wartosc)
  IF (wartosc AND &b00000001) = 0 THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagN()
  SetFlagH()
#ENDMACRO

#MACRO TestBit1(wartosc)
  IF (wartosc AND &b00000010) = 0 THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagN()
  SetFlagH()
#ENDMACRO

#MACRO TestBit2(wartosc)
  IF (wartosc AND &b00000100) = 0 THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagN()
  SetFlagH()
#ENDMACRO

#MACRO TestBit3(wartosc)
  IF (wartosc AND &b00001000) = 0 THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagN()
  SetFlagH()
#ENDMACRO

#MACRO TestBit4(wartosc)
  IF (wartosc AND &b00010000) = 0 THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagN()
  SetFlagH()
#ENDMACRO

#MACRO TestBit5(wartosc)
  IF (wartosc AND &b00100000) = 0 THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagN()
  SetFlagH()
#ENDMACRO

#MACRO TestBit6(wartosc)
  IF (wartosc AND &b01000000) = 0 THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagN()
  SetFlagH()
#ENDMACRO

#MACRO TestBit7(wartosc)
  IF (wartosc AND &b10000000) = 0 THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagN()
  SetFlagH()
#ENDMACRO

#MACRO ResetBit0(wartosc)  ' Reset bit 0 of wartosc
  wartosc = (wartosc) AND &b11111110
#ENDMACRO

#MACRO ResetBit1(wartosc)  ' Reset bit 1 of wartosc
  wartosc = (wartosc) AND &b11111101
#ENDMACRO

#MACRO ResetBit2(wartosc)  ' Reset bit 2 of wartosc
  wartosc = (wartosc) AND &b11111011
#ENDMACRO

#MACRO ResetBit3(wartosc)  ' Reset bit 3 of wartosc
  wartosc = (wartosc) AND &b11110111
#ENDMACRO

#MACRO ResetBit4(wartosc)  ' Reset bit 4 of wartosc
  wartosc = (wartosc) AND &b11101111
#ENDMACRO

#MACRO ResetBit5(wartosc)  ' Reset bit 5 of wartosc
  wartosc = (wartosc) AND &b11011111
#ENDMACRO

#MACRO ResetBit6(wartosc)  ' Reset bit 6 of wartosc
  wartosc = (wartosc) AND &b10111111
#ENDMACRO

#MACRO ResetBit7(wartosc)  ' Reset bit 7 of wartosc
  wartosc = (wartosc) AND &b01111111
#ENDMACRO

#MACRO SetBit0(wartosc)  ' Set bit 0 of wartosc
  wartosc = (wartosc) OR &b00000001
#ENDMACRO

#MACRO SetBit1(wartosc)  ' Set bit 1 of wartosc
  wartosc = (wartosc) OR &b00000010
#ENDMACRO

#MACRO SetBit2(wartosc)  ' Set bit 2 of wartosc
  wartosc = (wartosc) OR &b00000100
#ENDMACRO

#MACRO SetBit3(wartosc)  ' Set bit 3 of wartosc
  wartosc = (wartosc) OR &b00001000
#ENDMACRO

#MACRO SetBit4(wartosc)  ' Set bit 4 of wartosc
  wartosc = (wartosc) OR &b00010000
#ENDMACRO

#MACRO SetBit5(wartosc)  ' Set bit 5 of wartosc
  wartosc = (wartosc) OR &b00100000
#ENDMACRO

#MACRO SetBit6(wartosc)  ' Set bit 6 of wartosc
  wartosc = (wartosc) OR &b01000000
#ENDMACRO

#MACRO SetBit7(wartosc)  ' Set bit 7 of wartosc
  wartosc = (wartosc) OR &b10000000
#ENDMACRO


#MACRO ShiftRightSRL(n)
  REM  Shift n right into Carry (MSB set to 0), and set some flags.
  IF (n AND &b00000001) = 0 THEN
      ResetFlagC()  ' C contains old bit 0
    ELSE
      SetFlagC()
  END IF
  ResetFlagN()  ' reset flags N and H
  ResetFlagH()
  n = ((n SHR 1) AND &b01111111) ' Shift right by 1 and make sure to reset MSB
  IF (n = 0) THEN
      SetFlagZ()  ' Set or reset flag Z
    ELSE
      ResetFlagZ()
  END IF
#ENDMACRO


SUB RotateLeftCarry(BYREF NumRotate AS UBYTE)
  REM  Rotates NumRotate by 1 to the left through carry flag, and sets some Z80 flags.
  REM  This function is used both by RLA AND RL r instructions.
  STATIC AS UBYTE TempUbyteRLC
  TempUbyteRLC = (NumRotate AND &b10000000) ' Save old bit 7
  NumRotate = (NumRotate SHL 1)   ' shift left by 1
  IF (GetFlagC() <> 0) THEN NumRotate = (NumRotate OR &b00000001)  ' Rotate carry flag into bit 0
  IF (TempUbyteRLC = 0) THEN
      ResetFlagC()  ' Restore old bit 7 into carry flag
    ELSE
      SetFlagC()    ' Restore old bit 7 into carry flag
  END IF
  'IF NumRotate = 0 THEN SetFlagZ() ELSE ResetFlagZ()   ' Set or reset the Z flag
  REM I decided to do the same than the NO$GMB emulator: I always reset Z.
  ResetFlagZ()
  ResetFlagN()  ' Flags N & H should be reseted
  ResetFlagH()
END SUB


SUB RotateRightCarry(BYREF NumRotate AS UBYTE)
  REM  Rotates NumRotate by 1 to the right through carry flag, and sets some Z80 flags.
  REM  This function is used both by RRA and RR n instructions.
  STATIC AS UBYTE TempUbyte
  TempUbyte = (NumRotate AND &b00000001) ' Save old bit 0
  NumRotate = (NumRotate SHR 1)   ' shift right by 1
  IF GetFlagC() <> 0 THEN NumRotate = (NumRotate OR &b10000000)  ' Rotate carry flag into bit 7
  IF TempUbyte = 0 THEN
      ResetFlagC()  ' Restore old bit 0 into carry flag
    ELSE
      SetFlagC()    ' Restore old bit 0 into carry flag
  END IF
  'IF NumRotate = 0 THEN SetFlagZ() ELSE ResetFlagZ()   ' Set or reset the Z flag
  REM I decided to do the same than the NO$GMB emulator: I always reset Z.
  ResetFlagZ()
  ResetFlagN()  ' Flags N & H should be reseted
  ResetFlagH()
END SUB


SUB SbcA(BYREF ValToSub AS UBYTE)
  REM Subtract content of ValToSub along with the Carry flag from A, and sets some Z80 flags
  STATIC AS UBYTE UbyteBuff
  UbyteBuff = (Register.A - ValToSub)
  IF (GetFlagC() <> 0) THEN
    UbyteBuff -= 1
  END IF
  IF (Register.A < UbyteBuff) THEN
      SetFlagC()
    ELSE
      ResetFlagC()
  END IF
  IF (Register.A AND &b1111) < (UbyteBuff AND &b1111) THEN
      SetFlagH()
    ELSE
      ResetFlagH()
  END IF
  Register.A = UbyteBuff
  SetFlagN()
  IF (Register.A = 0) THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
END SUB


SUB AdcA(BYREF ValToAdd AS UBYTE)
  REM Add content of ValToAdd along with Carry flag to A, and sets some Z80 flags
  STATIC AS UBYTE UbyteBuff, FlagC
  IF GetFlagC() = 0 THEN FlagC = 0 ELSE FlagC = 1
  UbyteBuff = (Register.A + ValToAdd + FlagC)
  IF (UbyteBuff = 0) THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagN()
  IF (UbyteBuff < Register.A) THEN  ' Set flag C if carry from bit 7
      SetFlagC()
    ELSE
      ResetFlagC()
  END IF
  IF ((Register.A AND &b1111) + (ValToAdd AND &b1111) + FlagC) > 15 THEN  ' Set if carry from bit 3
      SetFlagH()
    ELSE
      ResetFlagH()
  END IF
  Register.A = UbyteBuff
END SUB


#MACRO ShiftSLA(ValueToShift)
  REM  Shift ValueToShift left by 1 bit, putting old bit 7 into Carry flag.
  IF (ValueToShift AND &b10000000) = 0 THEN
      ResetFlagC()
    ELSE
      SetFlagC()
  END IF
  ValueToShift = ((ValueToShift SHL 1) AND &b11111111)
  IF (ValueToShift = 0) THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagN()
  ResetFlagH()
#ENDMACRO


#MACRO ShiftSRA(ValueToShift)
  REM  Shift ValueToShift right by 1 bit, putting old bit 0 into Carry flag. MSB doesn't change.
  IF (ValueToShift AND &b00000001) = 0 THEN
      ResetFlagC()
    ELSE
      SetFlagC()
  END IF
  ValueToShift = (ValueToShift SHR 1)
  IF (ValueToShift AND &b01000000) = 0 THEN ' Restore old bit 7 (MSB) as it was before
      ValueToShift = (ValueToShift AND &b01111111)  ' reset MSB
    ELSE
      ValueToShift = (ValueToShift OR &b10000000)   ' set MSB
  END IF
  IF (ValueToShift = 0) THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagN()
  ResetFlagH()
#ENDMACRO


SUB RRCA()
  REM  Rotates A by 1 to the right, and sets some Z80 flags.
  STATIC AS UBYTE TempUbyte
  TempUbyte = (Register.A AND &b00000001) ' Save old bit 7
  Register.A = (Register.A SHR 1)   ' shift right by 1
  IF (TempUbyte = 0) THEN  ' if old bit 7 was 0
      ResetFlagC()  ' Reset carry flag
      Register.A = (Register.A AND &b01111111)  ' restore old bit 7 into bit 0
    ELSE  ' if old bit 7 was 1
      SetFlagC()    ' Set carry flag
      Register.A = (Register.A OR &b10000000)  ' restore old bit 7 into bit 0
  END IF
  REM I *do* touch the Z flag. The Z80 manual tells to not touch it (while the Gameboy
  REM CPU manual tells that it should be reset if result is 0...)
  REM I decided to do the same than the NO$GMB emulator: I always reset Z.
  'IF Register.A = 0 THEN SetFlagZ() ELSE ResetFlagZ()   ' Set or reset the Z flag
  ResetFlagZ()
  ResetFlagN()
  ResetFlagH()    ' Flags N & H should be reseted
END SUB


SUB RLCA()
  REM  Rotates register A by 1 to the left, and sets some Z80 flags.
  STATIC AS UBYTE TempUbyte
  TempUbyte = (Register.A AND &b10000000) ' Save old bit 7
  Register.A = (Register.A SHL 1)   ' shift left by 1
  IF TempUbyte = 0 THEN
      ResetFlagC()  ' Reset carry flag
      Register.A = (Register.A AND &b11111110)  ' restore old bit 7 into bit 0
    ELSE
      SetFlagC()    ' Set carry flag
      Register.A = Register.A OR &b00000001  ' restore old bit 7 into bit 0
  END IF
  REM I *do* touch the Z flag. The Z80 manual tells to not touch it (while the Gameboy
  REM CPU manual tells that it should be reset if result is 0...)
  REM I decided to do the same than the NO$GMB emulator: I always reset Z.
  'IF Register.A = 0 THEN SetFlagZ() ELSE ResetFlagZ()   ' Set or reset the Z flag
  ResetFlagZ()
  ResetFlagN()  ' Flags N & H should be reseted
  ResetFlagH()
END SUB


#MACRO RotateRLC(valtorot)
  REM The contents of register r are rotated left 1-bit position.
  REM The content of bit 7 is copied to the Carry flag and also to bit 0.
  IF (valtorot AND &b10000000) = 0 THEN   ' save bit7 into flag C
      ResetFlagC()
    ELSE
      SetFlagC()
  END IF
  valtorot = (valtorot SHL 1)
  REM now restore old bit 7 (holded in flag C) into bit 0
  IF GetFlagC() = 0 THEN
      valtorot = (valtorot AND &b11111110)
    ELSE
      valtorot = (valtorot OR &b00000001)
  END IF
  IF (valtorot = 0) THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagH()
  ResetFlagN()
#ENDMACRO


#MACRO RotateRRC(valtorot)
  REM The contents of register r are rotated right 1-bit position.
  REM The content of bit 0 is copied to the Carry flag and also to bit 7.
  IF (valtorot AND &b00000001) = 0 THEN   ' save bit0 into flag C
      ResetFlagC()
    ELSE
      SetFlagC()
  END IF
  valtorot = (valtorot SHR 1)
  REM now restore old bit 0 (holded in flag C) into bit 7
  IF (GetFlagC() = 0) THEN
      valtorot = (valtorot AND &b01111111)
    ELSE
      valtorot = (valtorot OR &b10000000)
  END IF
  IF (valtorot = 0) THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagH()
  ResetFlagN()
#ENDMACRO


#MACRO AdjustDAA()   ' Decimal adjust on register A, and sets some Z80 flags
  IF (GetFlagN() = 0) THEN   ' ADD, ADC or INC
      IF (GetFlagC() = 0) THEN
          IF (GetFlagH() = 0) THEN
              IF (Register.A AND &b00001111) < &h0A THEN
                  IF ((Register.A AND &b11110000) SHR 4) < &h0A THEN
                      ResetFlagC()
                    ELSE
                      Register.A += &h60
                      SetFlagC()
                  END IF
                ELSE
                  IF ((Register.A AND &b11110000) SHR 4) < &h09 THEN
                      Register.A += &h06
                      ResetFlagC()
                    ELSE
                      Register.A += &h66
                      SetFlagC()
                  END IF
              END IF
            ELSE
              IF ((Register.A AND &b11110000) SHR 4) < &h0A THEN
                  Register.A += &h06
                  ResetFlagC()
                ELSE
                  Register.A += &h66
                  SetFlagC()
              END IF
          END IF
        ELSE
          IF (GetFlagH() = 0) THEN
              IF (Register.A AND &b00001111) < &h0A THEN
                  Register.A += &h60
                ELSE
                  Register.A += &h66
              END IF
            ELSE
              Register.A += &h66
          END IF
          SetFlagC()
      END IF
    ELSE  ' SUB, SBC, DEC or NEG
      IF (GetFlagC() = 0) THEN
          IF (GetFlagH() = 0) THEN
              ResetFlagC()
            ELSE
              Register.A += &hFA
              ResetFlagC()
          END IF
        ELSE
          IF (GetFlagH() = 0) THEN
              Register.A += &hA0
            ELSE
              Register.A += &h9A
          END IF
          SetFlagC()
      END IF
  END IF
  IF (Register.A = 0) THEN
      SetFlagZ()
    ELSE
      ResetFlagZ()
  END IF
  ResetFlagH()
#ENDMACRO

