REM
REM --------------------------------------
REM This file is part of the zBoy project.
REM Copyright (C) Mateusz Viste 2010
REM --------------------------------------
REM
REM ROM-ONLY support
REM


SUB MBC0_MemoryRead(BYREF memaddr AS UINTEGER, BYREF Result AS UBYTE)
  IF (memaddr <= &h00FF) AND (BootRomEnabledFlag = 1) THEN
    Result = BootROM(memaddr)
  ELSEIF (memaddr < &h8000) THEN                            ' ROM memory [0000-7FFF]
    Result = MemoryROM(memaddr)
  ELSEIF (memaddr < &hA000) THEN                            ' Video RAM (8KiB) [8000-9FFF]
    Result = VideoRAM(memaddr)
  ELSEIF (memaddr >= &hC000) AND (memaddr < &hE000) THEN    ' Internal 8KiB RAM
    Result = MemoryInternalRAM(memaddr)
  ELSEIF (memaddr >= &hE000) AND (memaddr < &hFE00) THEN    ' RAM mirror
    Result = MemoryInternalRAM(memaddr - 8192)
  ELSEIF (memaddr >= &hFE00) AND (memaddr < &hFEA0) THEN    ' Sprite OAM memory
    Result = SpriteOAM(memaddr)
  ELSEIF (memaddr >= &hFF00) AND (memaddr <= &hFF4B) THEN   ' I/O registers
    Result = IoRegisters(memaddr)
  ELSEIF (memaddr >= &hFF80) THEN                           ' Hi RAM area [FF80-FFFF]
    Result = MemoryInternalHiRAM(memaddr)
  ELSE                                                  ' Else it's something I don't know about
    'IF (memaddr > &hFFFF) THEN
    '  PrintMsg("MMU FATAL ERROR!" & CHR(10) & "MEMORYREAD() -> INVALID MEMADDR! [" & HEX(memaddr) & "]") : PressAnyKey() : END
    'END IF
    'PRINT "MMU fatal error: Tried to read from an unknown memory address: " & HEX(memaddr) & "h"
    'SLEEP
    'QuitEmulator = 1
    Result = MemoryMAP(memaddr)
  END IF
END SUB


SUB MBC0_MemoryWrite(BYREF memaddr AS UINTEGER, BYREF DataByte AS UBYTE)
  IF (memaddr >= &hC000) AND (memaddr < &hE000) THEN        ' Internal 8KiB RAM
    MemoryInternalRAM(memaddr) = DataByte
  ELSEIF (memaddr >= &hE000) AND (memaddr < &hFE00) THEN    ' RAM mirror
    MemoryInternalRAM(memaddr - 8192) = DataByte
  ELSEIF (memaddr >= &hFE00) AND (memaddr < &hFEA0) THEN    ' Sprite OAM memory
    SpriteOAM(memaddr) = DataByte
  ELSEIF (memaddr >= &hFF80) AND (memaddr <= &hFFFF) THEN   ' Hi RAM area
    MemoryInternalHiRAM(memaddr) = DataByte
  ELSEIF (memaddr = &hFF41) THEN ' STAT register: Do not allow to write into 2 last bits of the STAT register, as these
    IoRegisters(&hFF41) = (IoRegisters(&hFF41) AND &b00000011) OR (DataByte AND &b11111100)    ' bits are the mode flag.
  ELSEIF (memaddr = &hFF44) THEN ' CURLINE [RW] Current Scanline.
    IoRegisters(memaddr) = 0     ' Writing into this register resets it.
    'UserMessage = "LY RESET"
  ELSEIF (memaddr = &hFF46) THEN ' Starts LCD OAM DMA transfer
    STATIC x AS INTEGER          '
    IF DataByte <= &hF1 THEN     ' Normally, should take 160us... but who cares?
        FOR x = 0 TO 159         ' Let's copy XX00-XX9F to FE00-FE9F
          MBC0_MemoryRead((DataByte SHL 8) + x, SpriteOAM(&hFE00 + x))
        NEXT x
      ELSE
        PrintMsg("MMU FATAL ERROR!" & CHR(10) & "TRIED TO RUN LCD OAM DMA TRANSFER USING AN INVALID RANGE!")
        PressAnyKey()
        END
    END IF
  ELSEIF(memaddr = &hFF04) THEN
    IoRegisters(&hFF04) = 0      ' Divide register: Writing any value sets it to 0
  ELSEIF (memaddr >= &hFF00) AND (memaddr <= &hFF4B) THEN   ' I/O registers
    IoRegisters(memaddr) = DataByte
  ELSEIF (memaddr = &hFF50) THEN
    BootRomEnabledFlag = 0
  ELSEIF (memaddr >= &h8000) AND (memaddr <= &h9FFF) THEN   ' Video RAM (8KiB)
    VideoRAM(memaddr) = DataByte
  'ELSEIF (memaddr >= &hFEA0) AND (memaddr <= &hFEFF) THEN   ' This area is not usable, but some games writes to it...
  '  REM                                                     ' For example Dr. Mario tries to write to FEFF... who knows why?
  'ELSEIF (memaddr >= &hFF4C) AND (memaddr <= &hFF7F) THEN   ' This area is not usable, but some games writes to it...
  '  REM                                                     ' For example Dr. Mario tries to write to FF7F... who knows why?
  ELSE                                                      ' Else it's something I don't know about
    'PRINT "MMU fatal error: Tried to write to an unknown memory address: "; HEX(memaddr)
    'QuitEmulator = 1
    MemoryMAP(memaddr) = DataByte
  END IF
END SUB
