REM
REM This file is part of the zBoy project
REM Copyright (C) Mateusz Viste 2010
REM

SUB ResetEmulator()  ' Resets all internal states of the emulator
  DIM AS UINTEGER x

  IF (BootRomEnabledFlag = 1) THEN
      Register.PC = &h000   ' The execution of code starts at 0x000 (Bootstrap ROM)
    ELSE
      Register.PC = &h100   ' The execution of code starts at 0x100 (Cartridge ROM)
  END IF
  FOR x = &h8000 TO &h9FFF  ' Reset all VRAM memory (this
    VideoRAM(x) = 0         ' is done by a real GameBoy as
  NEXT x                    ' well, via the bootstrap ROM)
  WriteRegBC(00, &h13)  ' 0013h
  WriteRegDE(00, &hD8)  ' 00D8h
  WriteRegHL(01, &h4D)  ' 014Dh
  Register.SP = &hFFFE  ' Stack pointer is inited at &hFFFE
  'MemoryWrite(&hFF05, &h00)   ' TIMA
  IoRegisters(&hFF05) = &h00   ' TIMA
  'MemoryWrite(&hFF06, &h00)   ' TMA
  IoRegisters(&hFF06) = &h00   ' TMA
  'MemoryWrite(&hFF07, &h00)   ' TAC
  IoRegisters(&hFF07) = &h00   ' TAC
  'MemoryWrite(&hFF10, &h80)   ' NR10
  IoRegisters(&hFF10) = &h80   ' NR10
  'MemoryWrite(&hFF11, &hBF)   ' NR11
  IoRegisters(&hFF11) = &hBF   ' NR11
  'MemoryWrite(&hFF12, &hF3)   ' NR12
  IoRegisters(&hFF12) = &hF3   ' NR12
  'MemoryWrite(&hFF14, &hBF)   ' NR14
  IoRegisters(&hFF14) = &hBF   ' NR14
  'MemoryWrite(&hFF16, &h3F)   ' NR21
  IoRegisters(&hFF16) = &h3F   ' NR21
  'MemoryWrite(&hFF17, &h00)   ' NR22
  IoRegisters(&hFF17) = &h00   ' NR22
  'MemoryWrite(&hFF19, &hBF)   ' NR24
  IoRegisters(&hFF19) = &hBF   ' NR24
  'MemoryWrite(&hFF1A, &h7F)   ' NR30
  IoRegisters(&hFF1A) = &h7F   ' NR30
  'MemoryWrite(&hFF1B, &hFF)   ' NR31
  IoRegisters(&hFF1B) = &hFF   ' NR31
  'MemoryWrite(&hFF1C, &h9F)   ' NR32
  IoRegisters(&hFF1C) = &h9F   ' NR32
  'MemoryWrite(&hFF1E, &hBF)   ' NR33
  IoRegisters(&hFF1E) = &hBF   ' NR33
  'MemoryWrite(&hFF20, &hFF)   ' NR41
  IoRegisters(&hFF20) = &hFF   ' NR41
  'MemoryWrite(&hFF21, &h00)   ' NR42
  IoRegisters(&hFF21) = &h00   ' NR42
  'MemoryWrite(&hFF22, &h00)   ' NR43
  IoRegisters(&hFF22) = &h00   ' NR43
  'MemoryWrite(&hFF23, &hBF)   ' NR??
  IoRegisters(&hFF23) = &hBF   ' NR??
  'MemoryWrite(&hFF24, &h77)   ' NR50
  IoRegisters(&hFF24) = &h77   ' NR50
  'MemoryWrite(&hFF25, &hF3)   ' NR51
  IoRegisters(&hFF25) = &hF3   ' NR51

  'MemoryWrite(&hFF26, &hF1)   ' NR52  / On GB only
  IoRegisters(&hFF26) = &hF1   ' NR52  / On GB only
  'MemoryWrite(&hFF26, &hF0)   ' NR52  / On SGB only
  'IoRegisters(&hFF26) = &hF0  ' NR52  / On SGB only

  'MemoryWrite(&hFF40, &h91)   ' LCDC
  IoRegisters(&hFF40) = &h91   ' LCDC
  'MemoryWrite(&hFF42, &h00)   ' SCY
  IoRegisters(&hFF42) = &h00   ' SCY
  'MemoryWrite(&hFF43, &h00)   ' SCX
  IoRegisters(&hFF43) = &h00   ' SCX
  'MemoryWrite(&hFF45, &h00)   ' LYC
  IoRegisters(&hFF45) = &h00   ' LYC
  'MemoryWrite(&hFF47, &hFC)   ' BGP
  IoRegisters(&hFF47) = &hFC   ' BGP
  'MemoryWrite(&hFF48, &hFF)   ' OBP0
  IoRegisters(&hFF48) = &hFF   ' OBP0
  'MemoryWrite(&hFF49, &hFF)   ' OBP1
  IoRegisters(&hFF49) = &hFF   ' OBP1
  'MemoryWrite(&hFF4A, &h00)   ' WY
  IoRegisters(&hFF4A) = &h00   ' WY
  'MemoryWrite(&hFF4B, &h00)   ' WX
  IoRegisters(&hFF4B) = &h00   ' WX

  'MemoryWrite(&hFFFF, &h00)   ' IE
  MemoryInternalHiRAM(&hFFFF) = &h00  ' IE

  Register.F = 0
  SetFlagZ()      ' Set some flags
  SetFlagH()      ' - as done by
  SetFlagC()      ' the NO$CASH
  Register.A = 1  ' GB emulator.
  InterruptsState = 0   ' IME is OFF at start (otherise "Hook" or "Pagemaster" don't even start, plus NO$GMB disables IME at start, too)
  HaltBug = 0    ' Emulation of the "HALT bug",
  HaltState = 0  ' must be all reset at start.
  TimerC = 0  ' Timer Counter
  DividerCycleCounter = 255  ' Divider counter
  EmulationStartTimer = TIMER    ' Save timer for later timing adjustements
  TotalCycles = 0                ' Reset CPU cycles counter
  'NextTimingCheck = 0           ' Reset NextTimingCheck (next time of speed adjustement)
  JoyNextCheck = 0               ' Reset JoyNextCheck (next time of checking Joypad states)
  JoyOldReg = 0
  JoyNewReg = 0
  CurRamBank = 0       ' Reset various
  Mbc1Model = 1        ' MMU-related
  Mbc1RamRomSelect = 0 ' variables
  IF RomInfos.MbcModel = 1 THEN ' On MBC1, set CurRomBank to 1
      CurRomBank = 1
    ELSE
      CurRomBank = 0
  END IF
  VideoClkCounterMode = 0     ' Reset some LCD variables
  VideoClkCounterVBlank = 0   ' LY is 0 at start (that's what NO$GMB does at least)
END SUB
