REM
REM Save & Load game state
REM This file is part of the zBoy project
REM Copyright (C) Mateusz Viste 2010
REM

' Here is the list of things I need to save (or load):
'  Register.A
'  Register.B
'  Register.C
'  Register.D
'  Register.E
'  Register.F
'  Register.H
'  Register.L
'  Register.PC
'  Register.SP
'  InterruptState
'  HaltBug
'  HaltState
'  TimerC
'  TotalCycles
'  NextTimingCheck
'  EmulationStartTimer
'  JoyOldReg
'  JoyNewReg
'  JoyNextCheck
'  KeybState
'  VideoClkCounterMode
'  VideoClkCounterVBlank
'  MemoryInternalRAM()
'  MemoryInternalHiRAM()
'  MemoryBankedRAM()  ' Check RAM size (if any) via RomInfos.RamSize
'  MemoryROM()        ' Check ROM size via RomInfos.RomSize
'  VideoRAM()
'  SpriteOAM()
'  IoRegisters()
'  MemoryMAP()
'  CurRamBank
'  CurRomBank
'  MbcModel
'  BootRomEnabledFlag = 0
'  DividerCycleCounter

SUB LoadGame()
  DIM AS INTEGER FileHandler, x
  DIM AS UINTEGER CrcSumFromFile
  DIM AS STRING FileHeader => "                "
  FileHandler = FREEFILE
  GetSaveDir()
  OPEN SaveDir & SaveStateFile() FOR BINARY AS #FileHandler
  GET #FileHandler,, FileHeader
  GET #FileHandler,, CrcSumFromFile
  IF (CrcSumFromFile <> RomInfos.CrcSum) OR (FileHeader <> "zBoy v0.11 SAVE.") OR (LOF(FileHandler) < 65535) THEN  ' Problem!
      UserMessage = "LOADING GAME STATE FAILED!"
    ELSE
      GET #FileHandler,, Register.A
      GET #FileHandler,, Register.B
      GET #FileHandler,, Register.C
      GET #FileHandler,, Register.D
      GET #FileHandler,, Register.E
      GET #FileHandler,, Register.F
      GET #FileHandler,, Register.H
      GET #FileHandler,, Register.L
      GET #FileHandler,, Register.PC
      GET #FileHandler,, Register.SP
      GET #FileHandler,, InterruptsState
      GET #FileHandler,, HaltBug
      GET #FileHandler,, HaltState
      GET #FileHandler,, TimerC
      GET #FileHandler,, TotalCycles
      GET #FileHandler,, EmulationStartTimer
      EmulationStartTimer = TIMER - (TotalCycles / 4194304)   ' Otherwise the CPU % gauge would become crazy!
      GET #FileHandler,, JoyOldReg
      GET #FileHandler,, JoyNewReg
      GET #FileHandler,, JoyNextCheck
      GET #FileHandler,, KeybState
      GET #FileHandler,, VideoClkCounterMode
      GET #FileHandler,, VideoClkCounterVBlank
      GET #FileHandler,, CurLY
      GET #FileHandler,, LastLY
      GET #FileHandler,, LastLYdraw
      GET #FileHandler,, MemoryInternalRAM()
      GET #FileHandler,, MemoryInternalHiRAM()
      FOR x = &hA000 TO (&hA000 + RomInfos.RamSize - 1)
        GET #FileHandler,, MemoryBankedRAM(x)  ' Check RAM size (if any) via RomInfos.RamSize
      NEXT x
      GET #FileHandler,, VideoRAM()
      GET #FileHandler,, SpriteOAM()
      GET #FileHandler,, IoRegisters()
      GET #FileHandler,, MemoryMAP()
      GET #FileHandler,, CurRamBank
      GET #FileHandler,, CurRomBank
      GET #FileHandler,, Mbc1Model
      GET #FileHandler,, Mbc1RamRomSelect
      GET #FileHandler,, BootRomEnabledFlag
      GET #FileHandler,, DividerCycleCounter
      UserMessage = "GAME LOADED"
  END IF
  CLOSE #FileHandler
END SUB


SUB SaveGame()
  DIM AS INTEGER FileHandler, x
  GetSaveDir()
  IF IsDirectory(SaveDir) = 0 THEN
    MKDIR SaveDir
  END IF
  FileHandler = FREEFILE
  OPEN SaveDir & SaveStateFile() FOR OUTPUT AS #FileHandler
  PUT #FileHandler,, "zBoy v0.11 SAVE."
  PUT #FileHandler,, RomInfos.CrcSum
  PUT #FileHandler,, Register.A
  PUT #FileHandler,, Register.B
  PUT #FileHandler,, Register.C
  PUT #FileHandler,, Register.D
  PUT #FileHandler,, Register.E
  PUT #FileHandler,, Register.F
  PUT #FileHandler,, Register.H
  PUT #FileHandler,, Register.L
  PUT #FileHandler,, Register.PC
  PUT #FileHandler,, Register.SP
  PUT #FileHandler,, InterruptsState
  PUT #FileHandler,, HaltBug
  PUT #FileHandler,, HaltState
  PUT #FileHandler,, TimerC
  PUT #FileHandler,, TotalCycles
  PUT #FileHandler,, EmulationStartTimer
  PUT #FileHandler,, JoyOldReg
  PUT #FileHandler,, JoyNewReg
  PUT #FileHandler,, JoyNextCheck
  PUT #FileHandler,, KeybState
  PUT #FileHandler,, VideoClkCounterMode
  PUT #FileHandler,, VideoClkCounterVBlank
  PUT #FileHandler,, CurLY
  PUT #FileHandler,, LastLY
  PUT #FileHandler,, LastLYdraw
  PUT #FileHandler,, MemoryInternalRAM()
  PUT #FileHandler,, MemoryInternalHiRAM()
  FOR x = &hA000 TO (&hA000 + RomInfos.RamSize - 1)
    PUT #FileHandler,, MemoryBankedRAM(x)  ' Check RAM size (if any) via RomInfos.RamSize
  NEXT x
  PUT #FileHandler,, VideoRAM()
  PUT #FileHandler,, SpriteOAM()
  PUT #FileHandler,, IoRegisters()
  PUT #FileHandler,, MemoryMAP()
  PUT #FileHandler,, CurRamBank
  PUT #FileHandler,, CurRomBank
  PUT #FileHandler,, Mbc1Model
  PUT #FileHandler,, Mbc1RamRomSelect
  PUT #FileHandler,, BootRomEnabledFlag
  PUT #FileHandler,, DividerCycleCounter
  CLOSE #FileHandler
  UserMessage = "GAME SAVED"
END SUB
